function lipsol(pname)
% LIPSOL   Linear programming Interior-Point Solver(s).
%          LIPSOL will prompt for a problem name to run.
%          LIPSOL('prob') will load prob.mat, prob.mps or prob.lpp;
%                or the compressed versions of these files, and then
%                run the problem.  If the problem name 'prob' is [],
%                the file default.mat will be loaded if it exists.

% Yin Zhang, January, 1995
% Department of Mathematics and Statistics
% University of Maryland  Baltimore County
% modified March, 2003, G. Sweriduk

global OUTFID

if (nargin == 0) 
    pname = input('Enter problem name: ','s'); 
end

t0 = cputime;

[A,b,c,lbounds,ubounds,BIG,NAME] = loadata(pname);

times(1) = cputime - t0;

t0 = cputime;

[A,b,c,lbounds,ubounds,FEASIBLE] = preprocess(A,b,c,lbounds,ubounds,BIG);

if ~FEASIBLE
   fprintf('\n'); 
   if isempty(OUTFID) 
       return; 
   end
   msginf =  'Infeasibility detected in preprocessing';
   fprintf(OUTFID, [pname '  0   ' msginf '\n']);
   return;
end;

[A,b,c,ubounds] = scaling(A,b,c,ubounds);
checkdense(A);

times(2) = cputime - t0;

t0 = cputime;
%c = -c;
%[x,y,z,s,w,info] = lipalg(A,b,c,ubounds);  % actual solver
[x,y,z,s,w,info] = miip(A,b,c,ubounds);  % actual solver
save results x y z s w info;

times(3) = cputime - t0;

t0 = cputime;

[xsol, objp] = postprocess(x,lbounds);

times(4) = cputime - t0;

save solution xsol;

printout(NAME,pname,info,objp,times);
%fprintf(1,'%c',7);  % ring a bell
